/** Something that we can write to. */
declare type Writable = {
	write(chunk: any): any
	end?(cb?: () => void): void
	close?(): Promise<void> | void
}
/**
 * Transform.
 * This is a helper for our transforms to be able to process the written log data more easily.
 * All the need to do is extend this class and add their own `format` method.
 * @param args forwarded to {@link Logger#setConfig}
 * @example
 * ``` javascript
 * import {inspect} from 'util'
 * import {Logger, Transform} from 'caterpillar'
 * class Pretty extends Transform {
 * 	format (entry) {
 * 		return inspect(entry, {colors: true})
 * 	}
 * }
 * new Logger()
 * 	.pipe(new Pretty())
 * 	.pipe(process.stdout)
 * 	.log('note', 'cool times', 5)
 * ```
 */
export declare class Transform implements Writable {
	/** Where is this Transform piping to? */
	private pipes
	/**
	 * Format the current entry representation.
	 * Your transformer should extend this.
	 */
	format(message: any): any
	/** Pipe future log entries into a caterpillar transform or a stream. */
	pipe<T extends Writable>(to: T): T
	/** Maintain a write queue such that multiple Deno writes do not stall */
	private writer
	/** Write to the child pipes. */
	write(chunk: any): Promise<void>
	/** Close the child pipes. */
	close(): Promise<void>
	end(cb?: () => void): void
}
export {}
//# sourceMappingURL=transform.d.ts.map
